-----------------------------------------
-- disparadors sobre la taula assignat --
-----------------------------------------

-----------------------------------------------------------------------------
-- les operacions ABM sobre la taula assignat disparen les actualitzacions --
-- de les dades estadistiques que tenen que vore amb assignacio de ----------
-- dispositius a usuaris: ---------------------------------------------------
---- est5 - donat un departament, nombre de dispositius en us ---------------
---- est6 - per a cada departament, valor economic de l-inventari -----------
---- est8 - departament que fa mes reassignacions ---------------------------
---- est9 - percentatge d-usuaris que no tenen cap dispositiu ---------------
---- est11 - departament amb mes tipus de dispositius -----------------------
---- est12 - nombre d-usuaris amb mes de cinq dispositius -------------------
-----------------------------------------------------------------------------

-------------------------------------------------------------------------
CREATE OR REPLACE FUNCTION actualitzar_est_assignat () RETURNS TRIGGER AS $$

DECLARE
  var_codi_departament character(10);
  var_any_actual integer;
  var_nombre_usuaris integer;
  var_usuaris_sense_dispositiu integer;
  quantitat integer;

BEGIN

-- actualitzem est5 ----------------------------------------------------------

-- esborrem la informacio antiga
  DELETE FROM "EST5";

---------------------------------------------------------------------------
-- para cada departament comptem la quantitat de dispositius que te
-- assignats un usuari que pertany a eixe departament.
-- Amb el DISTINC ens asegurem de no comptar varies vegades el mateix
-- dispositiu si esta assignat a mes d-un usuari
----------------------------------------------------------------------------
  INSERT INTO "EST5" (codi_departament, codi_tipus, nombre_disp)
  SELECT u.codi_departament, d.codi_tipus, COUNT(DISTINCT(a.codi_dispositiu))
    FROM usuari u, assignat a, dispositiu d
    WHERE u.codi_usuari = a.codi_usuari AND
          a.codi_dispositiu = d.codi_dispositiu
    GROUP BY u.codi_departament, d.codi_tipus;





-- actualitzem est6 ------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------
-- per aquesta estadistica entendre que quan es fa l-inventari anual, els dispositius que no s-inventarien
-- perque ja no estan disponibles, per exemple perque s-han avariat i no tenien reparacio o be perque han
-- desaparegut, es donen de baixa de la taula dispositiu. Per tant, considerare que corresponen a l-inventari
-- d-un departament tots aquells dispositius que, en el moment de la consulta, estan assignats a un usuari
-- d-eixe departament
---------------------------------------------------------------------------------------------------------------

-- esborrem la informacio antiga
  DELETE FROM "EST6";

-------------------------------------------------------------------
-- per a cada departament sumem el valor de tots els dispositius
-- que te assignats un usuari que pertany a eixe departament
-------------------------------------------------------------------
  INSERT INTO "EST6" (codi_departament, valor)
  SELECT u.codi_departament, SUM(d.valor)
  FROM dispositiu d, assignat a, usuari u
  WHERE d.codi_dispositiu = a.codi_dispositiu AND
        a.codi_usuari = u.codi_usuari
  GROUP BY u.codi_departament
  ORDER BY u.codi_departament;






-- actualitzem est8 ------------------------------------------------------------
----------------------------------------------------------------------------------
-- postgresql no permet l-anidacio de funcions d-agregacio per tant no podem fer
-- servir l-opcio mes logica que seria:
--   SELECT MAX(COUNT(*))
----------------------------------------------------------------------------------

-- nomes haurem d-actualitzar el registre corresponent a l-any actual

----------------------------------------------------------------------------------
-- copiem en una taula temporal tots els moviments de l-any actual, del tipus
-- reassignacio de dispositiu, agrupant-los per departament
----------------------------------------------------------------------------------

-- esborrem la taula temporal, si ja existeix
  DROP TABLE IF EXISTS taula_temporal;

---------------------------------------------------------------------------------
-- comptarem els moviments del tipus reassignacio que han hagut al any actual
-- i els enregistrarem a la taula temporal
----------------------------------------------------------------------------------

  CREATE TABLE taula_temporal AS
  SELECT codi_departament, COUNT(1) AS quantitat_moviments
  FROM moviments m, usuari u
  WHERE (m.codi_usuari_origen = u.codi_usuari OR
        m.codi_usuari_desti = u.codi_usuari) AND
        extract(year from m.data) = extract(year from current_date) AND
        m.tipus = 'REASSIGNACIO'
  GROUP BY u.codi_departament;


-- si ja existeix una fila per a l-any actual l-esborrarem
  DELETE FROM "EST8"
  WHERE "any" = extract(year from current_date);

-------------------------------------------------------------------
-- ara inserirem de nou el departament que te el maxim nombre
-- de moviments en la taula temporal (es a dir, del tipus
-- reassignacio)
-------------------------------------------------------------------
 
  INSERT INTO "EST8" ("any", codi_departament)
    SELECT extract(year from current_date), codi_departament
    FROM taula_temporal
    WHERE quantitat_moviments IN (
	                	 SELECT MAX(quantitat_moviments)
             		         FROM taula_temporal
		                 );

  DROP TABLE IF EXISTS taula_temporal;
 






-- actualitzem est9 ------------------------------------------------------------

-- esborrem les dades antigues
  DELETE FROM "EST9";

-- calculem el total d-usuaris de l-empresa
  SELECT COUNT(1) INTO var_nombre_usuaris FROM usuari;

-- mirem quants usuaris no tenen cap dispositiu assignat
  SELECT COUNT(1) INTO var_usuaris_sense_dispositiu
  FROM usuari
  WHERE codi_usuari NOT IN (
                           SELECT codi_usuari
                           FROM assignat                           );

-- calculem el percentatge i l-inserim a la taula
  INSERT INTO "EST9"(percentatge)
  VALUES (var_usuaris_sense_dispositiu * 100 / var_nombre_usuaris);







-- actualitzem est11 -------------------------------------------------------------
----------------------------------------------------------------------------------
-- postgresql no permet l-anidacio de funcions d-agregacio per tant no podem fer
-- servir l-opcio mes logica que seria:
--   SELECT codi_departament, MAX(COUNT(DISTINCT(td.codi_tipus)))
----------------------------------------------------------------------------------

-- esborrem les dades antigues
  DELETE FROM "EST11";
  
-- eliminen, si existeix, la taula temporal
  DROP TABLE IF EXISTS taula_temporal;

------------------------------------------------------------------------------
-- creem una taula temporal amb el nombre de tipus diferents de dispositius
-- (amb el DISTINCT eliminem tipus repetits)
-- que te cada departament
------------------------------------------------------------------------------
  CREATE TABLE taula_temporal AS
  SELECT u.codi_departament, COUNT(DISTINCT(td.codi_tipus)) AS quantitat
  FROM usuari u, assignat a, dispositiu d, tipus_dispositiu td
  WHERE u.codi_usuari = a.codi_usuari AND
        a.codi_dispositiu = d.codi_dispositiu AND
        d.codi_tipus = td.codi_tipus
  GROUP BY u.codi_departament;


------------------------------------------------------------------------------
-- dels registres de la taula temporal triem el departament que en te
-- una quantitat mes gran de tipus de dispositius (diferents)
------------------------------------------------------------------------------
  INSERT INTO "EST11" (codi_departament, quantitat)
  SELECT tt.codi_departament, tt.quantitat
  FROM taula_temporal tt
  WHERE tt.quantitat IN (
    SELECT MAX(tt.quantitat)
    FROM taula_temporal tt
    );

  DROP TABLE IF EXISTS taula_temporal;






-- actualitzem est12 -------------------------------------------------------------

-- esborrem la informacio antiga
  DELETE FROM "EST12";
  
------------------------------------------------------------
-- comptem quants usuaris compleixen la condicio de que al
-- comptar els dispositius que tenen assignats es mes gran
-- que cinq
------------------------------------------------------------
  INSERT INTO "EST12" (quant_users)
  (SELECT COUNT(1) FROM
  (SELECT COUNT(1)
         FROM usuari u, assignat a
         WHERE u.codi_usuari = a.codi_usuari
         GROUP BY u.codi_usuari
         HAVING COUNT(1) > 5) AS temp);

RETURN NULL;

END;
$$ LANGUAGE plpgsql;

CREATE TRIGGER disp_est_assignat
AFTER INSERT OR UPDATE OR DELETE ON assignat
    FOR EACH ROW EXECUTE PROCEDURE actualitzar_est_assignat ();







----------------------------------------
-- disparadors sobre la taula avaries --
----------------------------------------

-------------------------------------------------------------------------------
-- les operacions ABM sobre la taula avaries actualitzen les estadistiques ----
-- que tenen que vore amb les avaries sofertes pels dispositius. Es a dir: ----
---- est2 - portatils que no han sofert cap avaria des de la seua assignacio --
---- est13 - nombre d-avaries en un any a tota l-empresa ----------------------
---- est14 - top10 d-usuaris amb mes avaries ----------------------------------
-------------------------------------------------------------------------------

-------------------------------------------------------------------------
CREATE OR REPLACE FUNCTION actualitzar_est_avaries () RETURNS TRIGGER AS $$

DECLARE
  var_total_portatils integer;
  var_avaries_portatils integer;
  comptador integer;
  var_codi_proveidor character(10);

BEGIN

-- actualitzem est2 ----------------------------------------------------------

-- esborrem la informacio antiga
  DELETE FROM "EST2";

-----------------------------------------------------------------------------
-- no tindrem en compte els portatils que no estan assignats a ningun usuari
-- comptem quants portatils assignats n-hi ha i els anotem en una variable
-- amb DISTINCT evitarem els duplicats per un dispositiu estar assignat a
-- mes d-un usuari
-----------------------------------------------------------------------------
  SELECT COUNT(DISTINCT(a.codi_dispositiu)) INTO var_total_portatils
  FROM dispositiu d, assignat a
  WHERE d.codi_dispositiu = a.codi_dispositiu AND
        d.codi_tipus = 'TIP1000001';

--------------------------------------------------------------------------
-- per si de cas un dispositiu esta assignat a mes d-un usuari tindrem
-- amb compte unicament la darrera data d-assignacio
-- posarem aquesta informacio en una taula temporal
---------------------------------------------------------------------------

-- primer eliminem, si ja existeix, la taula temporal
  DROP TABLE IF EXISTS taula_temporal;

---------------------------------------------------------------------------
-- posarem a la taula temporal tots els dispositius (portatils) amb la
-- darrera data d-assignacio a un usuari
---------------------------------------------------------------------------
  CREATE TABLE taula_temporal AS
  SELECT codi_usuari, codi_dispositiu, MAX(data) AS data
  FROM assignat
  GROUP BY codi_usuari, codi_dispositiu;


--------------------------------------------------------------------------------
-- ara vorem quants han tingut alguna avaria posterior a la data d-assignacio
--------------------------------------------------------------------------------
  SELECT COUNT(1) INTO var_avaries_portatils
  FROM taula_temporal tt, dispositiu d, avaries av
  WHERE tt.codi_dispositiu = d.codi_dispositiu AND
        d.codi_dispositiu = av.codi_dispositiu AND
        d.codi_tipus = 'TIP1000001' AND
        d.avariat AND
        av.data_avaria >= tt.data;

  DROP TABLE IF EXISTS taula_temporal;

-----------------------------------------------
-- inserim el resultat a la taula estadistica
------------------------------------------------
  INSERT INTO "EST2" (percentatge)
  VALUES(100 - (100 * var_avaries_portatils / var_total_portatils));







-- actualitzem est13 ----------------------------------------------------------

-------------------------------------------------------
-- mirem si ja n-hi ha un registre per a l-any actual
-------------------------------------------------------
  SELECT COUNT(1) INTO comptador
  FROM "EST13" e
  WHERE e.any = extract(year from current_date);

------------------------------------------------------------------
-- si ja existeix un registre per a l-any actual incrementem
-- el valor del nombre d-avaries
-- Si no existeix, el creem amb el valor 1 per a nombre d-avaries
-----------------------------------------------------------------
  IF comptador > 0 THEN
    UPDATE "EST13" e
    SET nombre_avaries = nombre_avaries + 1
    WHERE e.any = extract(year from current_date);
  ELSE
    INSERT INTO "EST13" ("any", nombre_avaries)
    VALUES (extract(year from current_date), 1);
  END IF;






-- actualitzem est14 ----------------------------------------------------------

--esborrem la informacio antiga
  DELETE FROM "EST14";

---------------------------------------------------------------------------
-- ordenem totes les tuples de usuari mes dispositiu assignat amb avaria
-- i agafem els 10 que mes avaries tenen
----------------------------------------------------------------------------
  INSERT INTO "EST14" (codi_usuari)
  SELECT u.codi_usuari
  FROM usuari u, assignat ass, avaries ava
  WHERE u.codi_usuari = ass.codi_usuari AND
        ass.codi_dispositiu = ava.codi_dispositiu
  GROUP BY u.codi_usuari
  ORDER BY COUNT(1) DESC
  LIMIT 10;

RETURN NULL;

END;
$$ LANGUAGE plpgsql;

CREATE TRIGGER disp_est_avaries
AFTER INSERT OR UPDATE ON avaries
    FOR EACH ROW EXECUTE PROCEDURE actualitzar_est_avaries ();






-----------------------------------------------
-- disparadors sobre la taula linies_comanda --
-----------------------------------------------

-----------------------------------------------------------------------------------
-- les operacions sobre la taula linies_comanda nomes dispararan l-actualitzacio
-- de l-estadistica est3 - import de les comandes lligades a una ordre d-inversio
-- per any
-----------------------------------------------------------------------------------

-------------------------------------------------------------------------
CREATE OR REPLACE FUNCTION actualitzar_est_linies_comanda () RETURNS TRIGGER AS $$

DECLARE
  comptador integer;


BEGIN

-- actualitzem est3 ----------------------------------------------------------

-------------------------------------------------------------------
-- nomes actualitzarem el registre de l-any actual
-- comprovem doncs si ja existeix un registre per al any actual
-------------------------------------------------------------------
  SELECT COUNT(1) INTO comptador
  FROM "EST3" e3
  WHERE e3.any = extract(year from current_date);


-------------------------------------------------------------------------
-- si no existeix un registre per a l-any actual, el creem i posem en
-- import el valor de la linia de comanda inserida.
-- Si ja existira, aleshores incrementem el valor de import en el
-- de la linia de comanda inserida
-------------------------------------------------------------------------
  IF comptador = 0 THEN
    INSERT INTO "EST3" ("any", import)
      SELECT extract(year from c.data), SUM(lc.unitats * lc.preu_unitari)
      FROM comandes c, linies_comanda lc
      WHERE c.codi_comanda = lc.codi_comanda AND
            c.ordre_inversio
      GROUP BY extract(year from c.data)
      HAVING extract(year from c.data) = extract(year from current_date);
  ELSE
    UPDATE "EST3"
    SET import = (SELECT SUM(lc.unitats * lc.preu_unitari)
   	         FROM comandes c, linies_comanda lc
                 WHERE c.codi_comanda = lc.codi_comanda AND
                       c.ordre_inversio AND
                       extract(year from c.data) = extract(year from current_date));
  END IF;

RETURN NULL;

END;
$$ LANGUAGE plpgsql;

CREATE TRIGGER disp_est_linies_comanda
AFTER INSERT OR UPDATE OR DELETE ON linies_comanda
    FOR EACH ROW EXECUTE PROCEDURE actualitzar_est_linies_comanda ();






--------------------------------------------------
-- disparadors sobre la taula recepcio_material --
--------------------------------------------------

-----------------------------------------------------------------------------------
-- els moviments d-aquesta taula dispararan l-actualitzacio de les estadistiques
-- que tenen relacio l-entrada de dispositius a l-empresa. Es a dir:
---- est1 - total de dispositius comprats per any
---- est4 - proveidor amb un periode de lliurament mes curt
---- est10 - nombre d-unitats d-un dispositiu pendents d-assignacio
------------------------------------------------------------------------------------


-------------------------------------------------------------------------
CREATE OR REPLACE FUNCTION actualitzar_est_recepcio_material () RETURNS TRIGGER AS $$

DECLARE
  comptador integer;


BEGIN

-- actualitzem est1 ----------------------------------------------------------

-- entendrem que l-any de compra es quan es fa la recepcio del dispositiu

--------------------------------------------------------------------
-- nomes actualitzarem el registre de l-any actual
-- comprovem doncs si ja existeix un registre per a l-any actual
---------------------------------------------------------------------
  SELECT COUNT(1) INTO comptador
  FROM "EST1" e1
  WHERE "any" = extract(year from current_date);


-------------------------------------------------------------------------------------
-- si no existeix, l-inserim posant 1 en el camp del nombre de dispositius comprats
-- i si existeix incrementem el valor del camp en 1
--------------------------------------------------------------------------------------
  IF comptador = 0 THEN
    INSERT INTO "EST1" ("any", nombre_disps)
      VALUES (extract(year from current_date), 1);
  ELSE
    UPDATE "EST1" e1
    SET nombre_disps = nombre_disps + 1
    WHERE "any" = extract (year from current_date);
  END IF;







-- actualitzem est4 ----------------------------------------------------------

-- esborrem la informacio antiga
  DELETE FROM "EST4";

-- recalcularem els tems medis d-entrega i triarem el menor
-- PostGre no permet anidar funcions d-agregacio per tant farem servir una taula temporal

-- eliminem, si existeix, la taula temporal
  DROP TABLE IF EXISTS taula_temporal;


----------------------------------------------------------------------------------------
-- creem una taula temporal inserint en ella el temps mitja de lliurament de totes les 
-- recepcions de material de cada proveidor
-----------------------------------------------------------------------------------------
  CREATE TABLE taula_temporal AS (
                                 SELECT c.codi_proveidor, AVG(rm.temps_lliurament) AS temps_lliurament
                                 FROM comandes c, linies_comanda lc, recepcio_material rm
                                 WHERE c.codi_comanda = lc.codi_comanda AND
                                       lc.codi_comanda = rm.codi_comanda AND
                                       lc.numero_linia = rm.numero_linia
                                 GROUP BY c.codi_proveidor
                                 );


------------------------------------------------------------------------------------------
-- ara inserim el codi_proveidor del que tinga un temps_lliurament menor, a la taua "EST4"
-------------------------------------------------------------------------------------------
  INSERT INTO "EST4" (
                     SELECT codi_proveidor
                     FROM taula_temporal
                     WHERE temps_lliurament IN (
                                               SELECT MIN(temps_lliurament)
                                               FROM taula_temporal
                                               )
                     );

  DROP TABLE IF EXISTS taula_temporal;







-- actualitzem est10 ------------------------------------------------------------

-- esborrem la informacio antiga
 DELETE FROM "EST10";


---------------------------------------------------------------------------------
-- inserim a la taula estadistica el codi del tipus de dispositiu i les unitats
-- de tots els dispositiu, el codi del qual no existeix a la taula assignat
-- (es a dir, no esta assignat a cap usuari)
---------------------------------------------------------------------------------
  INSERT INTO "EST10" (codi_tipus, unitats)
    SELECT d.codi_tipus, COUNT(1)
    FROM dispositiu d
    WHERE d.codi_tipus NOT IN (
                              SELECT codi_tipus
                              FROM assignat
                              )
    GROUP BY d.codi_tipus;

RETURN NULL;

END;
$$ LANGUAGE plpgsql;

CREATE TRIGGER disp_est_recepcio_material
AFTER INSERT OR UPDATE OR DELETE ON recepcio_material
    FOR EACH ROW EXECUTE PROCEDURE actualitzar_est_recepcio_material ();

-------------------------------------------------------------------------------------

























